
#include "xparameters.h"
#include "stdio.h"
#include "xutil.h"
#include "xuartns550.h"
#include "xintc.h"
#include "xexception_l.h"

#define BS  8

XUartNs550      uart0 , uart1 ;
XIntc           intc ;
char            u0buf [ BS + 1 ] ;
char            u1buf [ BS + 1 ] ;


void handler ( void * callbackref , u32 event , unsigned eventdata )
{
    XUartNs550 *    uart = (XUartNs550 *) callbackref ;
    int             uart_num = ( uart == & uart0 ) ? 0 : 1 ;

    switch ( event )
    {
        case XUN_EVENT_RECV_DATA :
            printf ( "uart %d received %u.\n" , uart_num , eventdata ) ;
            if ( uart_num == 0 )
            {
                u0buf [ BS - 
                    uart -> ReceiveBuffer . RemainingBytes ] = '\0' ;
                printf ( "uart0: \"%s\"\n" , u0buf ) ;
                uart -> ReceiveBuffer . NextBytePtr = u0buf ;
            } else {
                u1buf [ BS - 
                    uart -> ReceiveBuffer . RemainingBytes ] = '\0' ;
                printf ( "uart1: \"%s\"\n" , u1buf ) ;
                uart -> ReceiveBuffer . NextBytePtr = u1buf ;
            }
            uart -> ReceiveBuffer . RemainingBytes = BS ;
            uart -> ReceiveBuffer . RequestedBytes = 0 ;
            break ;
        case XUN_EVENT_RECV_TIMEOUT :
            printf ( "uart %d rx timeout.\n" , uart_num ) ;
            break ;
        case XUN_EVENT_SENT_DATA :
            printf ( "uart %d send data.\n" , uart_num ) ;
            break ;
        case XUN_EVENT_RECV_ERROR :
            printf ( "uart %d rx error %u.\n" , uart_num , eventdata ) ;
            break ;
        case XUN_EVENT_MODEM :
            printf ( "uart %d modem event %u.\n" , uart_num , eventdata ) ;
            break ;
        default :
            printf ( "handler called with unknown event.\n" ) ;
            break ;
    }
}


int main ( void )
{
    char        buffer [ 20 ] ;

    printf ( "main() started.\n" ) ;
    XIntc_Initialize ( & intc , 0 ) ;
    XUartNs550_Initialize ( & uart0 , 0 ) ;
    XUartNs550_Initialize ( & uart1 , 1 ) ;
    printf ( "self test 0 return %d\n" , XUartNs550_SelfTest ( & uart0 ) ) ;
    printf ( "self test 1 return %d\n" , XUartNs550_SelfTest ( & uart1 ) ) ;

    uart0 . ReceiveBuffer . NextBytePtr = u0buf ;
    uart1 . ReceiveBuffer . NextBytePtr = u1buf ;
    uart0 . ReceiveBuffer . RemainingBytes = BS ;
    uart1 . ReceiveBuffer . RemainingBytes = BS ;
    uart0 . ReceiveBuffer . RequestedBytes = 0 ;
    uart1 . ReceiveBuffer . RequestedBytes = 0 ;

    XUartNs550_SetHandler ( & uart0 , handler , & uart0 ) ;
    XUartNs550_SetHandler ( & uart1 , handler , & uart1 ) ;
    XUartNs550_SetOptions ( & uart0 , XUN_OPTION_DATA_INTR | XUN_OPTION_MODEM_INTR ) ;
    XUartNs550_SetOptions ( & uart1 , XUN_OPTION_DATA_INTR | XUN_OPTION_MODEM_INTR ) ;
    XIntc_Connect ( & intc , XPAR_XPS_INTC_0_XPS_UART16550_0_IP2INTC_IRPT_INTR , 
            (XInterruptHandler) XUartNs550_InterruptHandler , & uart0 ) ;
    XIntc_Connect ( & intc , XPAR_XPS_INTC_0_XPS_UART16550_1_IP2INTC_IRPT_INTR , 
            (XInterruptHandler) XUartNs550_InterruptHandler , & uart1 ) ;
    XIntc_Start ( & intc , XIN_REAL_MODE ) ;
    XIntc_Enable ( & intc , XPAR_XPS_INTC_0_XPS_UART16550_0_IP2INTC_IRPT_INTR ) ;
    XIntc_Enable ( & intc , XPAR_XPS_INTC_0_XPS_UART16550_1_IP2INTC_IRPT_INTR ) ;
    XExc_Init () ;
    XExc_RegisterHandler ( XEXC_ID_NON_CRITICAL_INT , 
            (XExceptionHandler) XIntc_InterruptHandler , & intc ) ;
    XExc_mEnableExceptions ( XEXC_NON_CRITICAL ) ;
    printf ( "ready?\n" ) ;

    while ( 1 )
    {
    /*    fgets ( buffer , 16 , stdin ) ;
        printf ( "character %d: sending\n" , buffer [ 0 ] ) ;*/
        int i ;
        for ( i = 'A' ; i <= 'Z' ; i ++ )
        {
            buffer [ 0 ] = i ;
            XUartNs550_Send ( & uart1 , buffer , 1 ) ;
            buffer [ 0 ] = i + 32 ;
            XUartNs550_Send ( & uart0 , buffer , 1 ) ;
        }
    }

    return 0 ;

}





